<template>
  <div class="sleep-card" v-change-height:sleep-card-box v-if="!functype">
    <div class="sleep-card-box">
      <card-group :class="{ 'sleep-card-group': !card.hide }" v-for="(card, cardIndex) in cardList" :key="cardIndex">
        <!-- 按钮组 -->
        <template v-slot:content>
          <div class="card" v-if="!card.hide">
            <div class="card-item">
              <!-- 左边 -->
              <div class="card-item-left">
                <!-- 功能标识 -->
                <div class="card-item-left-main">
                  <!-- 图标 -->
                  <div class="card-item-left-main-icon iconfont" :class="`iconfont-${card.icon.key}`" />
                  <div class="card-item-left-main-text">
                    <!-- 卡片名字 -->
                    <p class="card-item-left-main-text-name" v-text="card.name" />
                  </div>
                </div>
              </div>
              <!-- 右边 -->
              <div class="card-item-right">
                <!-- 按钮 -->
                <gree-switch
                  class="card-item-right-switch"
                  :size="switchSize"
                  :class="{ on: card.icon.type === 'on' }"
                  :value="card.icon.type === 'on'"
                  @click="card.handler(card.key)"
                />
              </div>
            </div>
            <gree-block class="card-main">
              <gree-grid :border="false">
                <gree-radio-group v-model="sleepType">
                  <gree-radio-box :disabled="item.disabled" :name="item.name" v-for="(item, index) in sleepList" :key="index">
                    <gree-grid-item :icon="item.icon" :text="item.text" />
                  </gree-radio-box>
                </gree-radio-group>
              </gree-grid>
            </gree-block>
            <gree-block class="card-footer">
              <gree-radio-box
                v-model="currentPhysical"
                v-for="(item, index) in physicalList"
                :name="item.value - 1"
                :key="index"
                :label="item.text"
                v-show="SlpMod !== 3"
                :class="[`checked-${sleepType}`, { 'is-disabled': item.disabled }]"
              />
              <span class="edit-diy" @click="toEditDIY" v-show="SlpMod === 3">{{ '编辑DIY睡眠曲线' }}<gree-icon name="arrow-right" size="sm"/></span>
            </gree-block>
          </div>
        </template>
      </card-group>
    </div>
  </div>
</template>

<script>
import { Col, Row, Switch, Block, RadioBox, RadioGroup, Icon, Grid, GridItem } from 'gree-ui';
import { glyphs } from '@assets/iconfont/iconfont.json';
import BtnGroup from '@/components/BtnGroup/index';
import CardGroup from '@/components/CardGroup/index';
import { mapState, mapGetters, mapMutations, mapActions } from 'vuex';
import { sendDataToDevice } from '@PluginInterface';
import PopupGroup from '@/components/Popup/index';
import { types } from '@/store/types';

export default {
  name: 'SleepCard',
  components: {
    [Col.name]: Col,
    [Row.name]: Row,
    [Icon.name]: Icon,
    [Grid.name]: Grid,
    [Block.name]: Block,
    [Switch.name]: Switch,
    [GridItem.name]: GridItem,
    [BtnGroup.name]: BtnGroup,
    [RadioBox.name]: RadioBox,
    [CardGroup.name]: CardGroup,
    [RadioGroup.name]: RadioGroup,
    [PopupGroup.name]: PopupGroup
  },
  data() {
    return {
      selectSleepMod: 2,
      selectPhysical: 1,
      getSlpTimer: false, // 轮询标记
      checkTimer: null,
      disableUpdate: false,
      slpModExJson: [
        'SwhSlp',
        'SlpMod',
        'SmartSlpMod',
        'SmartSlpModEx',
        'StSlp1C',
        'StSlp1CInc',
        'StSlp1CSp',
        'StSlp1H',
        'StSlp1HInc',
        'StSlp1HSp',
        'StSlp2C',
        'StSlp2CInc',
        'StSlp2CSp',
        'StSlp2H',
        'StSlp2HInc',
        'StSlp2HSp',
        'StSlp3C',
        'StSlp3CInc',
        'StSlp3CSp',
        'StSlp3H',
        'StSlp3HInc',
        'StSlp3HSp',
        'StSlp4C',
        'StSlp4CInc',
        'StSlp4CSp',
        'StSlp4H',
        'StSlp4HInc',
        'StSlp4HSp'
      ],
      slpModExVal: [
        [1, 2, 4, 1, 1, 131, 1, 1, 131, 1, 120, 0, 6, 120, 0, 6, 300, 8, 6, 300, 8, 6, 0, 0, 1, 0, 0, 1],
        [1, 2, 4, 2, 1, 133, 1, 1, 133, 1, 120, 0, 6, 120, 0, 6, 300, 5, 6, 300, 5, 6, 0, 0, 1, 0, 0, 1],
        [1, 2, 4, 3, 1, 136, 1, 1, 136, 1, 120, 0, 6, 120, 0, 6, 300, 3, 6, 300, 3, 6, 0, 0, 1, 0, 0, 1],
        [1, 2, 4, 4, 1, 133, 1, 1, 133, 1, 240, 10, 6, 240, 10, 6, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1],
        [1, 2, 4, 5, 1, 138, 1, 1, 138, 1, 240, 10, 6, 240, 10, 6, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1],
        [1, 2, 4, 6, 1, 143, 1, 1, 143, 1, 240, 5, 6, 240, 5, 6, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1],
        [1, 2, 4, 7, 60, 130, 1, 60, 130, 1, 180, 0, 6, 180, 0, 6, 240, 5, 6, 240, 5, 6, 300, 10, 6, 300, 10, 6],
        [1, 2, 4, 8, 60, 133, 1, 60, 133, 1, 180, 0, 6, 180, 0, 6, 240, 5, 6, 240, 5, 6, 300, 10, 6, 300, 10, 6],
        [1, 2, 4, 9, 60, 133, 1, 60, 133, 1, 180, 0, 6, 180, 0, 6, 240, 5, 6, 240, 5, 6, 0, 0, 1, 0, 0, 1]
      ]
    };
  },
  computed: {
    ...mapState('control', {
      mac: state => state.mac,
      mqttVer: state => state.mqttVer,
      Mod: state => state.dataObject.Mod,
      SwhSlp: state => state.dataObject.SwhSlp,
      SlpMod: state => state.dataObject.SlpMod,
      SwUpDn: state => state.dataObject.SwUpDn,
      SmartSlpModEx: state => state.dataObject.SmartSlpModEx || 0,
      AntiDirectBlow: state => state.dataObject.AntiDirectBlow,
      functype: state => state.dataObject.functype
    }),
    ...mapGetters(['sleepCardDefine']),
    ...mapGetters('machine', ['statusMap', 'blindModelArr']),
    sleepList() {
      const result = [
        { name: 1, icon: this.sleepType === 1 && !this.cardDisabled ? 'baby-wash' : 'baby-wash-o', text: '儿童', disabled: this.cardDisabled },
        { name: 2, icon: this.sleepType === 2 && !this.cardDisabled ? 'teenager' : 'teenager-o', text: '青年', disabled: this.cardDisabled },
        { name: 3, icon: this.sleepType === 3 && !this.cardDisabled ? 'agedness' : 'agedness-o', text: '老年', disabled: this.cardDisabled },
        { name: 4, icon: this.sleepType === 4 && !this.cardDisabled ? 'like' : 'like-o', text: 'DIY', disabled: this.cardDisabled }
      ];
      return result;
    },
    cardDisabled() {
      return this.cardList[0].icon.type === 'off';
    },
    physicalList() {
      const result = [
        { value: 1, text: '偏寒体质', disabled: this.cardDisabled },
        { value: 2, text: '平和体质', disabled: this.cardDisabled },
        { value: 3, text: '偏热体质', disabled: this.cardDisabled }
      ];
      return result;
    },
    switchSize() {
      return Math.floor((window['lib'].flexible.rem / 108) * 88);
    },
    currentPhysical: {
      get: function() {
        if (this.SmartSlpModEx === 0) {
          return 1;
        }
        return (this.SmartSlpModEx - 1) % 3;
      },
      set: function(v) {
        this.setPhysical(v);
      }
    },
    sleepType: {
      get: function() {
        if (this.SlpMod === 0 && this.SwhSlp === 0) {
          return 99999;
        }
        if (this.SmartSlpModEx === 0) {
          return 0;
        }
        if (this.SlpMod === 3) {
          return 4;
        }
        return Math.ceil(this.SmartSlpModEx / 3);
      },
      set: function(v) {
        this.setSleepMod(v);
      }
    },
    cardList() {
      if (!Object.keys(this.statusMap).length) return [];
      const result = this.sleepCardDefine.map(model => {
        const identifier = model.identifier;
        if (!this.statusMap[identifier])
          return {
            icon: {},
            handler: () => {
              console.log('Undefined function');
            }
          };
        // 当前status定义
        const status = this.statusMap[identifier].status;
        // 取名称
        const defaultNameKey = `btn.${identifier}`;

        const statusNameText = status.name;
        const stateNameText = `${defaultNameKey}_${statusNameText}`;

        const name = stateNameText === this.$language(stateNameText) ? this.$language(defaultNameKey) : this.$language(stateNameText);
        // 图标
        const icon = glyphs.some(item => item.font_class === status.icon.key) ? status.icon : { key: 'undefined', type: 'off' };
        // 是否隐藏
        const hide = this.blindModelArr.includes(identifier);
        // 执行的方法
        const handler = (identifier, disable = false) => {
          disable || this.$stateMachine.nextStatus(identifier);
        };
        return { key: identifier, name, icon, hide, handler };
      });
      return result;
    },
    hasSmartSleep() {
      return Object.keys(this.statusMap).includes('SwhSlp');
    }
  },
  mounted() {
    if (this.SmartSlpModEx !== 0) {
      this.selectSleepMod = Math.ceil(this.SmartSlpModEx / 3);
    }
    if (this.mqttVer < 2 && this.hasSmartSleep) {
      this.getSlpVal(); // 轮询
      this.checkTimer = setInterval(() => {
        this.getSlpVal(); // 轮询
      }, 8000);
    }
    window.$bus.$on('clickSwitch', () => {
      if (this.selectSleepMod === 4) {
        this.setDiy();
      } else {
        this.sendSlpModEx();
      }
    });
  },
  methods: {
    ...mapMutations('control', {
      setChartData: 'SET_CHART_DATA',
      setDataObject: 'SET_DATA_OBJECT'
    }),
    ...mapMutations({
      setCheckObject: types.SET_CHECK_OBJECT
    }),
    ...mapActions({
      sendData: types.SEND_DATA,
      sendCtrl: types.SEND_CTRL
    }),
    toEditDIY() {
      this.$router.push('SleepDIY');
    },
    async getSlpVal() {
      if (this.disableUpdate) return;
      this.getSlpTimer = true;
      // data
      const cols = [
        'Slp1L1',
        'Slp1H1',
        'Slp1L2',
        'Slp1H2',
        'Slp1L3',
        'Slp1H3',
        'Slp1L4',
        'Slp1H4',
        'Slp1L5',
        'Slp1H5',
        'Slp1L6',
        'Slp1H6',
        'Slp1L7',
        'Slp1H7',
        'Slp1L8',
        'Slp1H8'
      ];
      const statueJson = JSON.stringify({
        mac: this.mac,
        t: 'status',
        cols
      });
      const res = await sendDataToDevice(this.mac, statueJson, false);
      const _res = JSON.parse(res);
      const dataObject = {};
      _res.forEach((item, index) => {
        dataObject[cols[index]] = item;
      });
      console.log('智眠轮循卡片-=--------------xxxx', dataObject);

      this.setDataObject(dataObject);
      this.setCheckObject(dataObject);
    },
    sendSlpModEx() {
      const index = (this.selectSleepMod - 1) * 3 + this.selectPhysical;
      const opt = this.slpModExJson;
      const p = this.slpModExVal[index];
      const setData = {};
      let pushData = {};
      if (!this.SwhSlp) {
        pushData = {
          WdSpd: 1,
          Quiet: 2,
          Tur: 0,
          SwUpDn: this.Mod === 4 ? this.SwUpDn : 0,
          AntiDirectBlow: this.Mod === 4 ? this.AntiDirectBlow : 1
        };
      }
      opt.forEach((json, index) => {
        setData[json] = p[index];
      });
      this.changeDataObject({ ...setData, ...pushData });
    },
    async changeDataObject(obj, hasToast = false) {
      clearInterval(this.checkTimer);
      // this.waitCallBack += 1;
      this.disableUpdate = true;
      this.getSlpTimer = false;
      const sendObj = { ...obj };
      const opt = Object.keys(sendObj);
      const p = Object.values(sendObj);
      const json = JSON.stringify({
        mac: this.mac,
        t: 'cmd',
        opt,
        p
      });
      console.log('changeDataObject====opt,t', opt, p);
      this.setDataObject(sendObj);
      this.setCheckObject(sendObj);
      const res = await sendDataToDevice(this.mac, json, true);
      const { r } = JSON.parse(res);
      console.log(JSON.parse(res));
      if (r === 200 && hasToast) {
        showToast(hasToast, 1);
      }
      // this.waitCallBack -= 1;
      // if (!this.waitCallBack) {
      if (this.mqttVer < 2) {
        setTimeout(() => {
          this.checkTimer = setInterval(() => {
            this.getSlpVal(); // 轮询
          }, 8000);
        }, 200);
      }
      // }
      this.disableUpdate = false;
    },
    setPhysical(v) {
      this.selectPhysical = v;
      this.sendSlpModEx();
    },
    setSleepMod(v) {
      if (v === this.sleepType) return;
      if (v < 4) {
        this.selectSleepMod = v;
        this.sendSlpModEx();
      } else {
        this.selectSleepMod = 4;
        this.setDiy();
      }
    },
    setDiy() {
      let setData = {};
      if (!this.SwhSlp) {
        setData = {
          SwhSlp: 1,
          SlpMod: 3,
          WdSpd: 1,
          Quiet: 2,
          Tur: 0,
          SwUpDn: this.Mod === 4 ? this.SwUpDn : 0,
          SmartSlpMod: 0,
          AntiDirectBlow: this.Mod === 4 ? this.AntiDirectBlow : 1
        };
      } else {
        setData = {
          SwhSlp: 1,
          SlpMod: 3,
          SmartSlpMod: 0
        };
      }
      let slpDiy = window.storage.get('slpDiy');
      if (!slpDiy) {
        slpDiy = {
          Slp1L1: 250,
          Slp1H1: 0,
          Slp1L2: 4,
          Slp1H2: 1,
          Slp1L3: 14,
          Slp1H3: 1,
          Slp1L4: 14,
          Slp1H4: 1,
          Slp1L5: 14,
          Slp1H5: 1,
          Slp1L6: 14,
          Slp1H6: 1,
          Slp1L7: 14,
          Slp1H7: 1,
          Slp1L8: 4,
          Slp1H8: 1
        };
      }
      setData = {
        ...setData,
        ...slpDiy
      };
      window.storage.set('slpDiy', slpDiy);
      this.changeDataObject(setData);
    }
  }
};
</script>

<style lang="scss">
.sleep-card {
  position: relative;
  transform: translateZ(1.1);
  .sleep-card-group {
    padding-bottom: 40px;
    .card {
      height: 610px;
      width: 100%;
      .card-item {
        // $cardPadding: 30px;
        // height: calc(160px - #{$cardPadding} * 2;
        height: 140px;
        width: 100%;
        display: flex;
        justify-content: space-between;
        align-items: center;
        // padding: $cardPadding 0;
        &-left {
          height: 100%;
          width: 100%;
          display: flex;
          justify-content: space-between;
          align-items: center;
          &-main {
            height: 100%;
            width: 100%;
            margin-left: 20px;
            display: flex;
            justify-content: flex-start;
            align-items: center;
            &-icon {
              $iconHeight: 68px;
              background-image: linear-gradient(to bottom, #abadfb, #9698eb);
              height: $iconHeight;
              width: $iconHeight;
              line-height: $iconHeight;
              margin-left: 42px;
              border-radius: 50%;
              font-size: 40px;
              color: #fff;
              text-align: center;
            }
            &-text {
              padding-left: 20px;
              &-name {
                font-size: 48px;
                color: #404657;
                display: inline;
              }
              &-describe {
                color: #989898;
                font-size: 36px;
                display: inline;
              }
              &-footer {
                font-size: 36px;
                color: #989898;
              }
            }
          }
          &::after {
            width: 0px;
          }
        }
        &-right {
          padding: 0 52px 0 24px;
          width: auto;
          display: flex;
          justify-content: flex-start;
          align-items: center;
          &-switch {
            background-color: #c5c5c5;
            &.on {
              background-color: #71acf1;
            }
          }
        }
      }
      .card-main {
        width: 100%;
        margin: 0;
        padding-left: 0;
        padding-right: 0;
        padding-bottom: 20px;
        // display: flex;
        // justify-content: space-around;
        .gree-grid {
          justify-content: center;
          .gree-check-base-box {
            padding: 0;
            border: none;
            width: 220px;
            height: 220px;
            margin: 0 13px;
            border-radius: 26px;
            background: rgba(152, 152, 152, 0.1);
            box-shadow: 0px 3px 24px 0px rgba(64, 70, 87, 0.03);
            &.is-disabled {
              opacity: 0.3;
            }
            &:nth-child(1) {
              .gree-icon {
                color: #8380c4;
              }
            }
            &:nth-child(2) {
              .gree-icon {
                color: #f5c77f;
              }
            }
            &:nth-child(3) {
              .gree-icon {
                color: #6da9f3;
              }
            }
            &:nth-child(4) {
              .gree-icon {
                color: #88e4b8;
              }
            }
            .gree-grid-item__text {
              font-size: 38px;
            }
            &.is-checked {
              &:nth-child(1) {
                background: linear-gradient(135deg, #afb0f0, #8380c4);
              }
              &:nth-child(2) {
                background: linear-gradient(135deg, #f9d49b, #f5c77f);
              }
              &:nth-child(3) {
                background: linear-gradient(135deg, #97c4fa, #6da9f3);
              }
              &:nth-child(4) {
                background: linear-gradient(135deg, #b9ecbb, #88e4b8);
              }
              .gree-icon,
              .gree-grid-item__text {
                color: #fff;
              }
            }
          }
        }
      }
      .card-footer {
        width: 100%;
        display: flex;
        align-items: center;
        justify-content: space-between;
        .gree-check-base-box {
          padding: 0;
          border: none;
          display: flex;
          align-items: center;
          justify-content: center;
          width: 276px;
          height: 80px;
          font-size: 44px;
          background: rgba(152, 152, 152, 0.1);
          border-radius: 40px;
          &.checked-0 {
            color: #404657;
          }
          &.is-checked.checked-1 {
            color: #fff;
            background: linear-gradient(90deg, #afb0f0, #8380c4);
          }
          &.is-checked.checked-2 {
            color: #fff;
            background: linear-gradient(90deg, #f9d49b, #f5c77f);
          }
          &.is-checked.checked-3 {
            color: #fff;
            background: linear-gradient(90deg, #97c4fa, #6da9f3);
          }
          &.is-disabled {
            opacity: 0.3;
            color: #404657;
            pointer-events: none;
            background: rgba(152, 152, 152, 0.1);
          }
        }
        .edit-diy {
          top: 10px;
          right: 30px;
          font-size: 40px;
          color: #71acf1;
          position: absolute;
          .gree-icon {
            opacity: 0.7;
            color: #989898;
            padding-left: 15px;
          }
        }
      }
    }
  }
}
</style>
